﻿namespace MefContrib.Models.Provider.Definitions.Configurable
{
    using System.Configuration;
    using System.Diagnostics;

    /// <summary>
    /// 
    /// </summary>
    public class PartCatalogConfigurationImport : ConfigurationElement
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PartCatalogConfigurationImport"/> class.
        /// </summary>
        public PartCatalogConfigurationImport()
        {
        }

        /// <summary>
        /// Gets or sets if default values are allowed.
        /// </summary>
        /// <value>A string containing the name of he contract.</value>
        /// <remarks>The default value is <see langword="false" />.</remarks>
        [ConfigurationProperty("allowDefault", DefaultValue = false, IsRequired = false)]
        public bool AllowDefault
        {
            [DebuggerStepThrough]
            get { return (bool)this["allowDefault"]; }
            [DebuggerStepThrough]
            set { this["allowDefault"] = value; }
        }

        /// <summary>
        /// Gets or sets the name of the contract.
        /// </summary>
        /// <value>A string containing the name of the contract.</value>
        [ConfigurationProperty("contractName", IsRequired = false),
            StringValidator(InvalidCharacters = " ~!@#$%^&*()[]{}/;'\"|\\")]
        public string ContractName
        {
            [DebuggerStepThrough]
            get { return this["contractName"] as string; }
            [DebuggerStepThrough]
            set { this["contractName"] = value; }
        }

        /// <summary>
        /// Gets or sets the type of the contract.
        /// </summary>
        /// <value>A string containing the type of the contract.</value>
        [ConfigurationProperty("contractType", IsRequired = false),
            StringValidator(InvalidCharacters = "~!@#$%^&*()[]{}/;'\"|\\")]
        public string ContractType
        {
            [DebuggerStepThrough]
            get { return this["contractType"] as string; }
            [DebuggerStepThrough]
            set { this["contractType"] = value; }
        }

        /// <summary>
        /// Gets or sets if default values are allowed.
        /// </summary>
        /// <value>A string containing the name of he contract.</value>
        /// <remarks>The default value is <see langword="false" />.</remarks>
        [ConfigurationProperty("isPrerequisite", DefaultValue = false, IsRequired = false)]
        public bool IsPrerequisite
        {
            [DebuggerStepThrough]
            get { return (bool)this["isPrerequisite"]; }
            [DebuggerStepThrough]
            set { this["isPrerequisite"] = value; }
        }

        /// <summary>
        /// Gets or sets if default values are allowed.
        /// </summary>
        /// <value>A string containing the name of he contract.</value>
        /// <remarks>The default value is <see langword="false" />.</remarks>
        [ConfigurationProperty("isRecomposable", DefaultValue = true, IsRequired = false)]
        public bool IsRecomposable
        {
            [DebuggerStepThrough]
            get { return (bool)this["isRecomposable"]; }
            [DebuggerStepThrough]
            set { this["isRecomposable"] = value; }
        }

        /// <summary>
        /// Gets or sets the name of the member.
        /// </summary>
        /// <value>A string containing the name of the member.</value>
        [ConfigurationProperty("member", IsRequired = true),
            StringValidator(InvalidCharacters = " ~!@#$%^&*()[]{}/;'\"|\\")]
        public string Member
        {
            [DebuggerStepThrough]
            get { return this["member"] as string; }
            [DebuggerStepThrough]
            set { this["member"] = value; }
        }

        /// <summary>
        /// Gets a collection of all the metadata defined for the <see cref="PartCatalogConfigurationImport"/>.
        /// </summary>
        /// <value>A <see cref="PartCatalogConfigurationMetadataCollection"/> object.</value>
        [ConfigurationProperty("metadata", IsDefaultCollection = false, IsRequired = false)]
        public PartCatalogConfigurationMetadataCollection Metadata
        {
            [DebuggerStepThrough]
            get { return (PartCatalogConfigurationMetadataCollection)base["metadata"]; }
        }
    }
}